/* ------------------------------------------------------------------------
 * $Id: InterceptionInjector.java 4 2007-04-06 04:33:54Z tpv $
 * Copyright 2007, Tim Vernum
 *
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the 
 * "Software"), to deal in the Software without restriction, including 
 * without limitation the rights to use, copy, modify, merge, publish, 
 * distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject 
 * to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ------------------------------------------------------------------------
 */

package net.sf.tie;

import org.aopalliance.intercept.MethodInterceptor;

/**
 * An <code>InterceptionInjector</code> wraps objects to support method interception.
 * 
 */
public interface InterceptionInjector
{
    /**
     * Wraps the designated object <code>endPoint</code> in any preconfigured <code>MethodInterceptors</code>.
     * Returns an object <i>O</i>, such that if a method <i>M</i> is invoked on <i>O</i>, then the provided
     * {@link MethodInterceptor interceptors} will be called, prior to an invocation of the original method <i>M</i> on <code>endPoint</code>.
     * It is acceptable for implementations to return the <code>endPoint</code> directly if there is no interception to perform, or if the
     * interception can be performed without constructing additional objects. However, it is typical for the return value to be a dynamically
     * created proxy around <code>endPoint</code>.
     * @param type The java type (typically an interface) which the returned object should implement.
     * @see ProxyInjector
     */
    public <T> T wrapObject(Class<T> type, T endPoint);

    /**
     * Determines whether the supplied <code>value</code> is a wrapped value from this interceptor.
     * <strong>NB:</strong> If <code>value</code> has been wrapped multiple times, then only the injector
     * that performed the last wrapping is required to return <code>true</code>. It is acceptible for all other
     * interceptors to return <code>false</code>.
     */
    public boolean isWrapped(Object value);

}
