/* ------------------------------------------------------------------------
 * $Id: InterceptorStack.java 4 2007-04-06 04:33:54Z tpv $
 * Copyright 2007, Tim Vernum
 *
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the 
 * "Software"), to deal in the Software without restriction, including 
 * without limitation the rights to use, copy, modify, merge, publish, 
 * distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject 
 * to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ------------------------------------------------------------------------
 */

package net.sf.tie;

import java.util.Collections;
import java.util.Set;

import org.aopalliance.intercept.MethodInterceptor;

/**
 * A utilty class for creating a {@link MethodInterceptorStack}.
 */
public final class InterceptorStack
{
    /**
     * Creates a stack containing a collection of {@link MethodInterceptor}
     * objects.<br>
     * The returned stack is implemented such that the
     * {@link MethodInterceptorStack#getInterceptors(Class, Object)} method
     * will always return <code>stack</code>.
     */
    public static MethodInterceptorStack stack(Iterable<MethodInterceptor> stack)
    {
        return new ListInterceptorStack(stack);
    }

    /**
     * Creates a stack containing a single {@link MethodInterceptor}.<br>
     * The returned stack is implemented such that the
     * {@link MethodInterceptorStack#getInterceptors(Class, Object)} method
     * will always return the single interceptor, <code>interceptor</code>.
     */
    public static MethodInterceptorStack singleton(MethodInterceptor interceptor)
    {
        return new ListInterceptorStack(Collections.singleton(interceptor));
    }

    /**
     * Creates an empty stack.<br>
     * The returned stack is implemented such that the
     * {@link MethodInterceptorStack#getInterceptors(Class, Object)} method
     * will always return an empty iterable.
     */
    public static MethodInterceptorStack empty()
    {
        Set<MethodInterceptor> set = Collections.emptySet();
        return new ListInterceptorStack(set);
    }

    private InterceptorStack()
    {
        // static class
    }
}
