/* ------------------------------------------------------------------------
 * $Id: MethodCall.java 25 2010-10-02 08:40:02Z tpv $
 * Copyright 2007, Tim Vernum
 *
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the 
 * "Software"), to deal in the Software without restriction, including 
 * without limitation the rights to use, copy, modify, merge, publish, 
 * distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject 
 * to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ------------------------------------------------------------------------
 */

package net.sf.tie;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;

/**
 * An encapsulation of a {@link Method} and its arguments. 
 * This class provides a conceptual simplification for classes
 *  that need to invoke methods via reflection.
 *  <p>
 *  <b>This class is not part of the public API for Tie</b>
 */
class MethodCall
{
    private final Method _method;
    private final Object[] _arguments;

    public MethodCall(Method method, Object... arguments)
    {
        _method = method;
        _arguments = arguments;
    }

    public Object[] getArguments()
    {
        return _arguments;
    }

    public Method getMethod()
    {
        return _method;
    }

    public Object invoke(Object on) throws Throwable
    {
        try
        {
            return _method.invoke(on, _arguments);
        }
        catch (InvocationTargetException ite)
        {
            // Unwrap any InvocationTargetExceptions to propagate the original exception
            throw ite.getCause();
        }
    }

    public String toString()
    {
        return getClass().getSimpleName()
                + "{ "
                + _method.getDeclaringClass().getSimpleName()
                + "."
                + _method.getName()
                + "("
                + Arrays.asList(_arguments)
                + ") }";
    }
}
