/* ------------------------------------------------------------------------
 * $Id: MethodInterceptorStack.java 4 2007-04-06 04:33:54Z tpv $
 * Copyright 2007, Tim Vernum
 *
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the 
 * "Software"), to deal in the Software without restriction, including 
 * without limitation the rights to use, copy, modify, merge, publish, 
 * distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject 
 * to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ------------------------------------------------------------------------
 */

package net.sf.tie;

import org.aopalliance.intercept.MethodInterceptor;

/**
 * The <code>MethodInterceptorStack</code> determines which {@link MethodInterceptor} implementations should be injected into each object.
 * @see InterceptorStack
 */
public interface MethodInterceptorStack
{
    /**
     * @param type The java type (typically an interface) which declares the methods to be intercepted.
     * @param endPoint The java object that provides an implementation of those methods.
     * @return A set of interceptors to apply over all invocations of methods on <code>endPoint</code>.
     */
    public <T> Iterable<MethodInterceptor> getInterceptors(Class<T> type, T endPoint);
}
