/* ------------------------------------------------------------------------
 * $Id: ProxyInjector.java 24 2008-02-03 10:13:12Z tpv $
 * Copyright 2007, Tim Vernum
 *
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the 
 * "Software"), to deal in the Software without restriction, including 
 * without limitation the rights to use, copy, modify, merge, publish, 
 * distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject 
 * to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ------------------------------------------------------------------------
 */

package net.sf.tie;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;

/**
 * An implementation of {@link InterceptionInjector} that uses Java dynamic proxies ({@link Proxy}).
 */
public class ProxyInjector implements InterceptionInjector
{
    private final MethodInterceptorStack _interceptors;

    public ProxyInjector(MethodInterceptorStack interceptors)
    {
        _interceptors = interceptors;
    }

    public <T> T wrapObject(Class<T> type, T endPoint)
    {
        InvocationHandler handler = new ProxyHandler(this, _interceptors.getInterceptors(type, endPoint), endPoint);
        Object proxy = Proxy.newProxyInstance(type.getClassLoader(), new Class[] { type }, handler);
        return type.cast(proxy);
    }

    public boolean isWrapped(Object value)
    {
        if (value == null)
        {
            return false;
        }
        
        if (!Proxy.isProxyClass(value.getClass()))
        {
            return false;
        }

        InvocationHandler handler = Proxy.getInvocationHandler(value);
        if (handler instanceof ProxyHandler)
        {
            ProxyHandler proxy = (ProxyHandler) handler;
            return proxy.getInjector() == this;
        }

        return false;
    }

}
