/* ------------------------------------------------------------------------
 * $Id: WrappedMethodInvocation.java 4 2007-04-06 04:33:54Z tpv $
 * Copyright 2007 Tim Vernum
 * All rights reserved
 * ------------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to  deal in  the Software without  restriction, including
 * without limitation  the rights to  use, copy, modify,  merge, publish,
 * distribute, and/or sell copies of  the Software, and to permit persons
 * to whom  the Software is furnished  to do so, provided  that the above
 * copyright notice(s) and this permission notice appear in all copies of
 * the  Software and  that both  the above  copyright notice(s)  and this
 * permission notice appear in supporting documentation.
 * 
 * THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
 * EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR  A PARTICULAR PURPOSE AND NONINFRINGEMENT
 * OF  THIRD PARTY  RIGHTS. IN  NO EVENT  SHALL THE  COPYRIGHT  HOLDER OR
 * HOLDERS  INCLUDED IN  THIS  NOTICE BE  LIABLE  FOR ANY  CLAIM, OR  ANY
 * SPECIAL INDIRECT  OR CONSEQUENTIAL DAMAGES, OR  ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS  OF USE, DATA OR PROFITS, WHETHER  IN AN ACTION OF
 * CONTRACT, NEGLIGENCE  OR OTHER TORTIOUS  ACTION, ARISING OUT OF  OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * Except as  contained in  this notice, the  name of a  copyright holder
 * shall not be used in advertising or otherwise to promote the sale, use
 * or other dealings in this Software without prior written authorization
 * of the copyright holder.
 * ------------------------------------------------------------------------
 */

package net.sf.tie;

import java.lang.reflect.Method;

import org.aopalliance.intercept.Joinpoint;
import org.aopalliance.intercept.MethodInvocation;

/**
 * Extends the <a href="http://aopalliance.sf.net">AOP Alliance</a>'s
 * {@link MethodInvocation} interface with the concept of a method implementation.
 * <br>
 * The {@link #getMethodImplementation() method implementation} is the method as it is defined
 * on the class (object) being intercepted. This may the same as the
 * {@link #getMethodDeclaration() declared method}, if the declaring class is the same as the 
 * implementing class, but will in normal usage, not be. Typical usage of method
 * interception in <i>Tie</i> is for intercepting via interfaces - in this situation the
 * declared method has no implementation, and so the two {@link Method} objects cannot be
 * the same.
 * <br>
 * The common use case that requires knowledge of the implementing method is annotation 
 * processing. There are situations where the presence (or absence) of a particular 
 * annotation on the method implementation will affect the manner in which interception
 * is applied.
 */
public interface WrappedMethodInvocation extends MethodInvocation
{
    /**
     * This method exists for orthogonality with {@link #getMethodImplementation()},
     * but will always return the same value as {@link MethodInvocation#getMethod()}.
     */
    public Method getMethodDeclaration();

    /**
     * @return A method defined on {@link Joinpoint#getThis()}, that is the implementation
     * for {@link #getMethodDeclaration()}.
     */
    public Method getMethodImplementation();
}
